package FeldmanVerifiableSecretSharing.ShamirSecretSharing;

import java.math.BigInteger;

/**
 * Created by Tzlil on 1/28/2016.
 *
 * container of lagrange polynomial
 *
 * can't be constructed, constructor is private
 *
 * l = (image/divisor)* polynomial
 *
 * Note : image and divisor stored separately for avoiding lose of information by division
 */
class LagrangePolynomial{
    public final Polynomial polynomial;
    public final BigInteger image;
    public final BigInteger divisor;

    /**
     * inner constructor, stores all given parameters
     * @param polynomial
     * @param image
     * @param divisor
     */
    private LagrangePolynomial(Polynomial polynomial, BigInteger image, BigInteger divisor) {
        this.polynomial = polynomial;
        this.image = image;
        this.divisor = divisor;
    }

    /**
     * static method
     * @param points array points s.t there are no couple of points that shares the same x value
     *
     * @return the lagrange polynomials that mach to given points
     *
     * @throws Exception there exists i != j s.t points[i].x == points[j].x
     */
    public static LagrangePolynomial[] lagrangePolynomials(Polynomial.Point[] points) throws Exception {
        LagrangePolynomial[] lagrangePolynomials = new LagrangePolynomial[points.length];
        Polynomial[] factors = new Polynomial[points.length];
        for (int i = 0 ; i < factors.length ; i++){
            factors[i] = new Polynomial(new BigInteger[]{BigInteger.ZERO.subtract(points[i].x),BigInteger.ONE}); // X - Xi
        }
        Polynomial product;
        BigInteger divisor;
        for(int i = 0; i < points.length; i ++) {
            product = Polynomial.ONE;
            divisor = BigInteger.ONE;
            for (int j = 0; j < points.length; j++) {
                if (i != j) {
                    divisor = divisor.multiply(points[i].x.subtract(points[j].x));
                    product = product.mul(factors[j]);
                }
            }
            if(divisor.equals(BigInteger.ZERO))
                throw new Exception();
            lagrangePolynomials[i] = new LagrangePolynomial(product,points[i].y,divisor);
        }
        return lagrangePolynomials;
    }
}
